"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.render = void 0;
/**
 * ditaa API:
 *     https://github.com/stathissideris/ditaa
 */
const path = require("path");
const utility = require("./utility");
const compute_checksum_1 = require("./lib/compute-checksum");
const CACHE = {};
/**
 * Render ditaa diagrams with `code` to svg.
 * @param code the ditaa code
 * @param args args passed to ditaa.jar
 * @return rendered svg
 */
function render(code = "", args = []) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const info = yield utility.tempOpen({
                prefix: "mume_ditaa",
                suffix: ".ditaa",
            });
            yield utility.writeFile(info.fd, code);
            const dest = (yield utility.tempOpen({
                prefix: "mume_ditaa",
                suffix: ".svg",
            })).path;
            let codes = code;
            if (args.length > 0) {
                codes = args.join() + code;
            }
            const checksum = compute_checksum_1.default(codes);
            if (checksum in CACHE &&
                CACHE[checksum].code === code &&
                utility.isArrayEqual(args, CACHE[checksum].args)) {
                // already rendered
                return CACHE[checksum].svg;
            }
            yield utility.execFile("java", [
                "-Djava.awt.headless=true",
                "-jar",
                path.resolve(utility.extensionDirectoryPath, "./dependencies/ditaa/ditaa.jar"),
                info.path,
                dest,
                "--svg",
            ].concat(args));
            const pathToSvgWithoutVersion = dest.replace(/\?[\d\.]+$/, "");
            const svg = yield utility.readFile(pathToSvgWithoutVersion);
            // save to cache
            CACHE[checksum] = { code, args, svg };
            return svg;
        }
        catch (error) {
            throw new Error(`Java is required to be installed.\n${error.toString()}`);
        }
    });
}
exports.render = render;
//# sourceMappingURL=ditaa.js.map